@@ -350,11 +350,12 @@ or specify that directory with --msgdir=PATH''')
 
     @staticmethod
     def set(msg, fieldInfo, value, index=0):
-        if("int" in fieldInfo.type):
+        if("int" in fieldInfo.type or "enumeration" == fieldInfo.type):
             if isinstance(value, str):
                 value = value.strip()
                 if value.startswith("0x"):
                     value = int(value, 0)
+        if("int" in fieldInfo.type):
             value = int(float(value))
         elif("float" in fieldInfo.type):
             value = float(value)
@@ -474,6 +475,15 @@ class BitFieldInfo(object):
         self.count=1
         self.enum=enum
         self.idbits=idbits
+        # add a couple fields from decorators of the 'get' function
+        self.offset = int(get.offset)
+        self.size = int(get.size)
+
+    def exists(self, msg, index=0):
+        return self.parent.exists(msg, index)
+
+    def end_location(self, index=0):
+        return self.parent.end_location(index)
 
 class FieldInfo(object):
     def __init__(self, name, type, units, minVal, maxVal, description, get, set, count, bitfieldInfo, enum, idbits=0):
@@ -489,3 +499,18 @@ class FieldInfo(object):
         self.bitfieldInfo=bitfieldInfo
         self.enum=enum
         self.idbits=idbits
+        # add a couple fields from decorators of the 'get' function
+        self.offset = int(get.offset)
+        self.size = int(get.size)
+        # give our bitfields a reference to us
+        for bfi in self.bitfieldInfo:
+            bfi.parent = self
+
+    def exists(self, msg, index=0):
+        end_of_field = self.offset + self.size * (index+1)
+        if msg.hdr.GetDataLength() >= end_of_field:
+            return True
+        return False
+    
+    def end_location(self, index=0):
+        return self.offset + self.size * (index+1)
