@@ -94,16 +94,23 @@ def transformInt(field, value):
     if "Scale" in field or "Offset" in field:
         scale = fieldItem(field, "Scale", 1.0)
         offset = fieldItem(field, "Offset", 0.0)
-        value = value * scale + offset
+        value = float(value) * float(scale) + float(offset)
     return value
 
 def fieldMin(field):
     if fieldIsInt(field):
         numBits = fieldNumBits(field)
-        if fieldIsSigned(field):
-            minVal = -2**(numBits-1)
+        # if scale is negative, reverse min/max based on range of storage value
+        if "Scale" in field and float(field['Scale']) < 0.0:
+            if fieldIsSigned(field):
+                minVal = 2**(numBits-1)-1
+            else:
+                minVal = 2**numBits-1
         else:
-            minVal = 0
+            if fieldIsSigned(field):
+                minVal = -2**(numBits-1)
+            else:
+                minVal = 0
         minVal = transformInt(field, minVal)
     elif "Type" in field and field["Type"] == 'float64':
         minVal = "DBL_MIN"
@@ -115,10 +122,17 @@ def fieldMin(field):
 def fieldMax(field):
     if fieldIsInt(field):
         numBits = fieldNumBits(field)
-        if fieldIsSigned(field):
-            maxVal = 2**(numBits-1)-1
+        # if scale is negative, reverse min/max based on range of storage value
+        if "Scale" in field and float(field['Scale']) < 0.0:
+            if fieldIsSigned(field):
+                maxVal = -2**(numBits-1)
+            else:
+                maxVal = 0
         else:
-            maxVal = 2**numBits-1
+            if fieldIsSigned(field):
+                maxVal = 2**(numBits-1)-1
+            else:
+                maxVal = 2**numBits-1
         maxVal = transformInt(field, maxVal)
     elif "Type" in field and field["Type"] == 'float64':
         maxVal = "DBL_MAX"
@@ -389,6 +403,19 @@ def Structs(inputData):
             structList.update(Structs(data))
     return structList
 
+def NestedArraysExist(field, structs, outer_array_exists):
+    if fieldCount(field) > 1:
+        if outer_array_exists:
+            return True
+        outer_array_exists = True
+    if field["Type"] in structs:
+        s = structs[field["Type"]]
+        for subfield in s['Fields']:
+            found_sub_array = NestedArraysExist(subfield, structs, outer_array_exists)
+            if found_sub_array:
+                return True
+    return False
+    
 # this replaces fields that are references to structs with the fields from the referenced struct,
 # and also sets a few items of field meta-data (Location, StructSize, Count)
 def PatchStructs(inputData):
@@ -406,17 +433,31 @@ def PatchStructs(inputData):
                     location = 0
                     outfields = []
                     for field in msg['Fields']:
+                        nested_structs = NestedArraysExist(field, structs, False)
+                        if nested_structs:
+                            #print("found nested struct in %s.%s" % (msg["Name"], field["Name"]))
+                            if fieldCount(field) > 1:
+                                # flatten array!
+                                #print("Flattening array %s[%d]" % (field["Name"], fieldCount(field)))
+                                for j in range(fieldCount(field)):
+                                    fieldcopy = copy.deepcopy(field)
+                                    fieldcopy['Name'] = fieldcopy['Name'] + "_" + str(j)
+                                    fieldcopy['Count'] = 1
+                                    outfields.append(fieldcopy)
+                                continue
                         if field["Type"] in structs:
                             s = structs[field["Type"]]
-                            # Compute the size of the structure and store it in each field in the structure.
-                            # This is useful for finding subsequent elements of fields of arrays of structs,
-                            # because we'll treat them as parallel arrays in the generated code, not as
-                            # hierarchical structs.
+                            # For arrays of structs, compute the size of the structure and store it in each
+                            # field in the structure.  This is useful for finding subsequent elements of
+                            # fields of arrays of structs, because we'll treat them as parallel arrays in
+                            # the generated code, not as hierarchical structs.
                             structSize = 0
                             for subfield in s['Fields']:
                                 if resolve_sizes_and_locations:
                                     try:
-                                        structSize += fieldSize(subfield)
+                                        fs = fieldSize(subfield) * fieldCount(subfield)
+                                        if fieldCount(field) > 1 or fieldCount(s) > 1:
+                                            structSize += fs
                                     except KeyError:
                                         # if we have a KeyError when trying to get fieldSize,
                                         # it's because we have a nested Struct that hasn't been
@@ -429,20 +470,32 @@ def PatchStructs(inputData):
                             subfieldLocation = 0
                             for subfield in s['Fields']:
                                 subfieldcopy = copy.deepcopy(subfield)
-                                subfieldcopy['Name'] = field['Name'] + "_" + subfield['Name']
+                                #  if a field name was specified, prepend it to the names of the sub-fields
+                                # of the struct.  Also allow an unnamed field so that nothing is prepended.
+                                if 'Name' in field:
+                                    subfieldcopy['Name'] = field['Name'] + "_" + subfield['Name']
+                                else:
+                                    subfieldcopy['Name'] = subfield['Name']
                                 if resolve_sizes_and_locations:
-                                    subfieldcopy['StructSize'] = structSize
+                                    if structSize != 0:
+                                        subfieldcopy['StructSize'] = structSize
                                     subfieldcopy['Location'] = location + subfieldLocation
-                                # inherit count from parent field, so that if it's an array,
-                                # we'll be an array too.
-                                subfieldcopy['Count'] = fieldCount(field)
+                                # subfield count should be greater of parent field count and subfield type count
+                                # error if both are > 1
+                                if fieldCount(subfieldcopy) == 1:
+                                    if fieldCount(field) > 1:
+                                        subfieldcopy['Count'] = fieldCount(field)
+                                else:
+                                    if fieldCount(field) > 1:
+                                        raise MessageException("ERROR! %s Field %s Count %d > 1 and Subfield %s Count %d > 1" % (msg["Name"], field['Name'], fieldCount(field), subfieldcopy['Name'], fieldCount(subfieldcopy)))
+                                        subfieldcopy['Count'] = fieldCount(field) * fieldCount(subfieldcopy)
                                 #TODO How to handle array of bitfields?!?
                                 if "Bitfields" in subfieldcopy:
                                     for bits in subfieldcopy["Bitfields"]:
                                         bits['Name'] = field['Name'] + "_" + bits['Name']
                                 outfields.append(subfieldcopy)
                                 if resolve_sizes_and_locations:
-                                    subfieldLocation += fieldSize(subfield)
+                                    subfieldLocation += fieldSize(subfield) * fieldCount(subfield)
                             if resolve_sizes_and_locations:
                                 # add size of struct times struct count to running total of field location
                                 location += subfieldLocation * fieldCount(field)
